-- User: Krakyn
-- Created: 11/30/2015 5:58 PM
local _G = _G
Bam = LibStub('AceAddon-3.0'):NewAddon('Bam', 'AceConsole-3.0', 'AceEvent-3.0')

local B = _G.Bam
local db
local guid = UnitGUID("player")
local MAJOR, MINOR = "Bam Mod Redux v.1", ""
local LOADING = true
local LAST_BAM = 0
local defaults = {
  cherry = true,
  enabled = true,
  bam_sound_enable = true,
  bam_sound = "Sound/character/PlayerExertions/TaurenFemaleFinal/TaurenFemaleFinalDeathA.ogg",
  bam_string = 'Bam! \[%s - *%d\*]',
  chat_throttle = true,
  chat_throttle_rate = 4,
  bam_threschold = 30000,
  custom_channel_enable = false,
  custom_channel = "",
  whisper_enable = false,
  whisper_to = "",
  channels = {
    ['PRINT'] = true,
    ['EMOTE'] = true,
    ['SAY'] = false,
    ['YELL'] = false,
    ['RAID'] = false,
    ['PARTY'] = false,
    ['INSTANCE_CHAT'] = false,
    ['BATTLEGROUND'] = false,
    ['RAID_WARNING'] = false,
    ['GUILD'] = false,
    ['OFFICER'] = false,
  }
}

local sound_bank = {
  ["Sound/character/PlayerExertions/TaurenFemaleFinal/TaurenFemaleFinalDeathA.ogg"] = "Mcjiggs",
  ["Interface/Addons/Bam/Sound/bam.mp3"] = "Bam!",
  ["Interface/Addons/Bam/Sound/baaaam.ogg"] = "Baaam",
  ["Interface/Addons/Bam/Sound/crow.mp3"] = "Crow",
  ["Interface/Addons/Bam/Sound/finishhim.ogg"] = "Finish Him",
  ["Interface/Addons/Bam/Sound/headshot.ogg"] = "Headshot",
  ["Interface/Addons/Bam/Sound/mouthpop.mp3"] = "Mouth Pop",
  ["Interface/Addons/Bam/Sound/Quack.ogg"] =  "Quack",
  ["Interface/Addons/Bam/Sound/tritt.ogg"] =  "Tritt",
  ["Interface/Addons/Bam/Sound/vollgas.ogg"] = "Vollgas",
  ["Interface/Addons/Bam/Sound/yehgurl.mp3"] = "Yeh Gurl"
} -- TODO: let LibSharedMedia add to this

local c_names = {
  ['PRINT'] = 'Silent (print)',
  ['SAY'] = 'Say',
  ['EMOTE'] = 'Emote',
  ['YELL'] = 'Yell',
  ['PARTY'] = 'Party (/p)',
  ['RAID'] = 'Raid (/ra)',
  ['RAID_WARNING'] = 'Raid Warning (/rw)',
  ['GUILD'] = 'Guild (/g)',
  ['OFFICER'] = 'Officer (/o)',
  ['INSTANCE_CHAT'] = 'Instance (/i)',
  ['BATTLEGROUND'] = 'Battleground (/bg)',
  ['CHANNEL'] = 'Chat Channel (name or number)',
  ['WHISPER'] = 'Whisper',
}

local KEYWORD_COLOR = '|cFFFFFF00'
local RESUME_COLOR = '|cFF44FFFF'
local CLEAR = '|r'
local L = {
  ['Cherry'] = 'This is probably your first time installing BaM or you lost the config table in the depths of your cavernous ass. Either way, type |cFFFFFF00/bam|r to begin. You will never see this message again, hopefully.',
  ['Bam Format'] = [[Global Message Format]],
  ['Bam Format Syntax'] = RESUME_COLOR.."Keywords:"..CLEAR.."\n"..KEYWORD_COLOR.."%s"..CLEAR.." - Spell name\n"..KEYWORD_COLOR.."%d"..CLEAR.." - Damage\n"..KEYWORD_COLOR.."(%b)"..CLEAR.." - Blocked\n "..KEYWORD_COLOR.."(%a)"..CLEAR.." - Absorbed\n"..KEYWORD_COLOR.."(%o)"..CLEAR.." - Overkill\n"..KEYWORD_COLOR.."%("..CLEAR..", "..KEYWORD_COLOR.."%)"..CLEAR.." - Non-conditional parenthesis\n".. KEYWORD_COLOR.."%%"..CLEAR.. " - Percent Sign"
}

--- all the general tasks involved with changes to each setting
local optfuncs = {
  enabled = {
    onChange = function(self, t, k, v)
      if v == true then
        B:RegisterEvent('COMBAT_LOG_EVENT_UNFILTERED')
        B:RegisterEvent('CHAT_MSG_CHANNEL_NOTICE')
        B:Print('Enabled')
        if not LOADING then
          PlaySoundFile(db.bam_sound)
        end
      else
        B:UnregisterAllEvents()
        B:Print('Disabled.')
      end
    end
  },
  custom_channel_enable = {
    onChange = function(self, t, k, v)
      if v == true then
        B:CHAT_MSG_CHANNEL_NOTICE()
      end
    end
  }
}

--- used to ensure that in-game option changes are handled consistently
local function UpdateOpt(k, v)
  if v ~= nil then
    db[k] = v
    --print('updating', k, 'to', tostring(v))
  else
    v = db[k]
    --print('handling', k)
  end
  if optfuncs[k] then
    if type(optfuncs[k].onChange) == 'function' then
      optfuncs[k].onChange(B, db, k, v)
    else
      B:Print('Bad option handler for '..k)
    end
  end
end

--- generate channel config data
local function ChannelOpt(chat_type, order)
  return {
    type = 'toggle',
    name = c_names[chat_type],
    set = function(info, value)
      db.channels[chat_type] = value
    end,
    get = function(info)
      return db.channels[chat_type]
    end,
    order = order
  }
end

function B:OnInitialize()
  if (not _G.BamDB) then
    _G.BamDB = defaults
  end
  db = _G.BamDB

  if db.cherry then
    self:Print(L['Cherry'])
    db.cherry = nil
  end

  local opts = {
    type='group',
    name='Bam!',
    set = function(info, value)
      UpdateOpt(info[#info], value)
    end,
    get = function(info)

      if not db[info[#info]] then
        return defaults[info[#info]]
      else
        return db[info[#info]]
      end
    end,

    args = {
      enabled = {
        type = 'toggle',
        name = '|cFF00FF00Enable Bam output',
        order = 1,
        width = 'full',
      },
      bam_sound_enable = {
        type = 'toggle',
        name = 'Play Sound File',
        order = 10,
      },
      bam_sound = {
        type = 'select',
        name = 'Audio',
        values = sound_bank,
        set = function(info, value)
          --print('Bam', 'setting some shit', value)
          --print('Bam', PlaySoundFile(tostring(value), "Master"))
          db.bam_sound = value
        end,
        order = 15,
      },
      bam_string = {
        type = 'input',
        name = L['Bam Format'],
        desc = L['Bam Format Syntax'],
        order = 100,
        width = 'full'
      },
      chat_throttle = {
        type='toggle',
        name = 'Throttle Bam Outputs',
        desc = 'Set a minimum delay between output messages.',
        order = 200
      },
      bam_threschold = {
        type='input',
        name='Damage Threschold',
        desc='Crits below this amount won\'t be output.',
        order = 210,
      },
      chat_throttle_rate = {
        type = 'range',
        name = 'Throttle Rate',
        order = 230,
        min = 0,
        max = 60,
        softMax = 60,
        step = 0.1,
        width='full',
      },
      t_channels = {
        type = 'header',
        name = 'Outputs',
        order = 300,
      },
      c_say = ChannelOpt('SAY', 400),
      c_emote = ChannelOpt('EMOTE', 405),
      c_yell = ChannelOpt('YELL', 410),
      c_party = ChannelOpt('PARTY', 415),
      c_raid = ChannelOpt('RAID', 420),
      c_instance = ChannelOpt('INSTANCE_CHAT', 425),
      c_battleground = ChannelOpt('BATTLEGROUND', 430),
      c_raidwarning = ChannelOpt('RAID_WARNING', 435),
      c_guild = ChannelOpt('GUILD', 440),
      c_officer = ChannelOpt('OFFICER', 450),
      c_print = ChannelOpt('PRINT', 455),
      t_custom_channel = {
        type = 'header',
        name = 'Custom Channel',
        order = 500,
      },
      custom_channel_enable = {
        type = 'toggle',
        name = '|cFFFF6600Enable|r',
        order = 517,
      },
      custom_channel = {
        type = 'input',
        name = 'Channel (Name of Number)',
        width='double',
        order = 515,
        set = function(_, value)
          db.channels.CHANNEL = value
        end,
        get = function()
          return db.channels.CHANNEL
        end,
      },
      t_custom_tell = {
        type = 'header',
        name = 'Whisper',
        order = 520,
      },
      whisper_enable = {
        type = 'toggle',
        name = '|cFFFF6600Enable|r',
        order = 527,
      },
      whisper_to = {
        type = 'input',
        name = 'Player Name (Realm-Name or Name)',
        order = 525,
        width='double'
      },
      t_warn = {
        type = 'description',
        name = 'Flooding public channels or players can get your account |cFFFFFF00suspended|r. Use caution when enabling channel or whisper output.',
        fontSize = 'large',
        order = 600
      }
    }
  }

  -- run through option handlers
  self:Print(MAJOR,MINOR)
  for k,_ in pairs(db) do
    UpdateOpt(k)
  end


  LibStub("AceConfig-3.0"):RegisterOptionsTable("Bam", opts)
  self.BlizOptions = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Bam")
  self:RegisterChatCommand("bam", "BamCommand", true)

  LOADING = nil
end

function B:BamCommand(input)
  local opt =  self:GetArgs(input, 1)
  if not opt then
    InterfaceOptionsFrame_OpenToCategory(self.BlizOptions) InterfaceOptionsFrame_OpenToCategory(self.BlizOptions)
  elseif opt =='on' or opt == 'enable' then
    UpdateOpt('enabled', true)
  elseif opt == 'off' or opt == 'disable' then
    UpdateOpt('enabled', false)
  end
end

function B:Fire(spell, amount, overkill, blocked, absorbed)
  local bam_msg
  bam_msg = db['bam_string']
  bam_msg = string.gsub(bam_msg, '%%%(', '<paren>')
  bam_msg = string.gsub(bam_msg, '%%%)', '<cparen>')
  bam_msg = string.gsub(bam_msg, '%((.*%%o.*)%)', function(a)
    if overkill > 0 then
      return string.gsub(a, '%%%o', overkill)
    else
      return ''
    end
  end)
  bam_msg = string.gsub(bam_msg, '%((.*%%a.*)%)', function(a)
    if absorbed > 0 then
      return string.gsub(a, '%%%a', absorbed)
    else
      return ''
    end
  end)
  bam_msg = string.gsub(bam_msg, '%((.*%%b.*)%)', function(a)
    if blocked > 0 then
      return string.gsub(a, '%%%b', blocked)
    else
      return ''
    end
  end)
  bam_msg = string.gsub(bam_msg, '<paren>', '(')
  bam_msg = string.gsub(bam_msg, '<cparen>', ')')
  bam_msg = string.gsub(bam_msg, '%%s', tostring(spell))
  bam_msg = string.gsub(bam_msg, '%%d', tostring(amount))
  bam_msg = string.gsub(bam_msg, '%%%%', '%')

  for chat_type, enabled in pairs(db.channels) do
    if enabled then
      if db.bam_sound_enable == true then
        PlaySoundFile(db.bam_sound, "Master")
      end
      if chat_type == 'PRINT' then
        print(bam_msg)
      elseif chat_type == 'CHANNEL' then
        if  db.custom_channel_enable  and self.active_chats[db.custom_channel] then
          local channel = self.active_chats[db.custom_channel]
          --print(chat_type, channel)
          SendChatMessage(bam_msg, chat_type, nil, channel)
        end
      elseif chat_type == 'WHISPER' then
        if db.whisper_enable then
          local channel = db.whisper_to
          SendChatMessage(bam_msg, chat_type, nil, channel)
        end
      else
        --print(chat_type, channel)
        SendChatMessage(bam_msg, chat_type, nil, channel)
      end
    end
  end
end

function B:CHAT_MSG_CHANNEL_NOTICE()
  B.active_chats = {}
  local a = B.active_chats
  local channels = {GetChannelList() }
  local index, name
  for i, value in ipairs(channels) do
    if i % 2 == 1 then
      index = value
    else
      name = value
      a[tostring(index)] = tonumber(index)
      a[tostring(name)] = tonumber(index)
    end
  end
end

local highest_bam = {'', 0, -1, -1, -1, false}
function B:COMBAT_LOG_EVENT_UNFILTERED(e, timestamp, combatEvent, ...)
  if combatEvent:match("_DAMAGE") then
    local sourceguid = select(2,...)
    if sourceguid ~= guid then
      return
      end
    local spellName = select(11,...)
    local critical = select(19,...)
    local amount, overkill = select(13,...)
    local blocked, absorbed = select(17,...)
    if (critical == true) then
      -- threschold
      if amount < db.bam_threschold then
        return
      end

      -- throttling
      if db.bam_throttle == true then
        if amount > highest_bam[2] then
          highest_bam = {spellName, amount, overkill, blocked, absorbed, critical}
          if GetTime() - LAST_BAM < db.bam_throttle_rate then
            return
          else
            LAST_BAM = GetTime()
            spellName, amount, overkill, blocked, absorbed, critical = unpack(highest_bam)
            highest_bam = {'', 0, -1, -1, -1, false}
          end
        end
      end

      self:Fire(spellName, amount, overkill, blocked, absorbed, critical)
    end
  end
end